import * as THREE from 'https://unpkg.com/three@v0.150.0/build/three.module.js';

var renderer, scene, camera;
var chopper;

// Two variables that hold our shader code. This code will be sent to the GPU.
var cubeVertexShader, cubeFragmentShader;

/**
 * Function to tell us the milliseconds
 */
function millis() {
    return (new Date()).getTime();
}

/**
 * Converts degrees to radians
 */
function toRad(degree) {
    return Math.PI * 2 * degree / 360;
}

window.onLoad = async () => {
    var canvasContainer = document.getElementById('myCanvasContainer');
    var width = 800;
    var height = 500;

    // We will load the shader code with 'fetch'
    cubeVertexShader = await (await fetch('vertex.glsl')).text();
    cubeFragmentShader = await (await fetch('fragment.glsl')).text();

    renderer = new THREE.WebGLRenderer();
    renderer.setSize(width, height);
    canvasContainer.appendChild(renderer.domElement);

    scene = new THREE.Scene();

    // We will use a perspective camera, with FOV 80 degrees, correct aspect ratio, near plane at distance 1, far plane at distance 1000
    camera = new THREE.PerspectiveCamera(80, width / height, 1, 1000);
    camera.position.z = 15;
    camera.position.y = 2;
    camera.up = new THREE.Vector3(0, 1, 0);
    camera.lookAt(new THREE.Vector3(0, 0, 0));
    scene.add(camera);

    scene.add(createHangar());
    chopper = createChopper();
    scene.add(chopper);

    draw();
}

function draw() {
    requestAnimationFrame(draw);

    //--Task-- Copy the rotation code from the previous task here.

    renderer.render(scene, camera); //We render our scene with our camera
}

/**
 * Create the hangar with walls.
 * Hangar currently consists of 5 walls which have different colors.
 */
function createHangar() {
    var hangar = new THREE.Object3D();
    var halfPi = Math.PI / 2;

    var leftWall = createWall(0x555555);
    leftWall.position.set(-10, 0, 0);
    leftWall.rotation.set(0, halfPi, 0);
    hangar.add(leftWall);

    var rightWall = createWall(0x333333);
    rightWall.position.set(10, 0, 0);
    rightWall.rotation.set(0, -halfPi, 0);
    hangar.add(rightWall);

    var backWall = createWall(0x444444);
    backWall.position.set(0, 0, -10);
    hangar.add(backWall);

    var ceiling = createWall(0x111111);
    ceiling.position.set(0, 10, 0);
    ceiling.rotation.set(halfPi, 0, 0);
    hangar.add(ceiling);

    var floor = createWall(0x222222);
    floor.position.set(0, -10, 0);
    floor.rotation.set(-halfPi, 0, 0);
    hangar.add(floor);

    return hangar;
}

/**
 * This function creates the chopper.
 * --Task-- Copy the code from the previous task here.
 */
function createChopper() {
    var chopper = new THREE.Object3D();
    var body = createCube(0xeeeeee);
    chopper.add(body);

    //... Copy whatever you did here in the previous task.

    return chopper;
}

/**
 * Creates a cube.
 * --Task-- Finish this function.
 */
function createCube(colorCode) {
    var geometry = new THREE.BoxGeometry(2, 2, 2); // Vertices shared across one cube face only!

    // The THREE.Color class allows some operations to be done with the color
    // Let us say that a dark color is half of the color given to the function
    var darkColor = new THREE.Color(colorCode).multiplyScalar(0.5);

    // --Task-- Create also the lightColor object.

    /*  There are 2 types of variables we can send to the GPU:
        Uniform variable - a variable that is the same for all vertices and fragments
        Attribute variable - a variable that has a specific value for each vertex
        In this task we will use attribute variables to specify colors for the vertices.
    */

    /*  --Task--
        Attribute variables are used like this:
        1) Create a Float32Array (typed JS array) with a correct size, eg: (vertexCount * elementsPerVertexCount)
        2) Assign correct values to each of the elements in the typed array (use a for loop here)
        3) Call something like: geometry.setAttribute("variableName", new THREE.BufferAttribute(yourTypedArray, elementsPerVertexCount));
    */

    // ShaderMaterial wants the shader codes and values for uniform variables (which we do not use here).
    var material = new THREE.ShaderMaterial({
        vertexShader: cubeVertexShader,    // Assign shader codes
        fragmentShader: cubeFragmentShader
    });

    var cube = new THREE.Mesh(geometry, material); // We create our cube with that material

    return cube;
}

/**
 * Creates a wall using the plane geometry.
 */
function createWall(colorCode) {
    var geometry = new THREE.PlaneGeometry(20, 20, 1);
    var material = new THREE.MeshBasicMaterial({ color: colorCode });
    var wall = new THREE.Mesh(geometry, material);

    return wall;
}
